//*--------------------------------------------------------------------------------------
//*      ATMEL Microcontroller Software Support  -  ROUSSET  -
//*--------------------------------------------------------------------------------------
//* The software is delivered "AS IS" without warranty or condition of any
//* kind, either express, implied or statutory. This includes without
//* limitation any warranty or condition with respect to merchantability or
//* fitness for any particular purpose, or against the infringements of
//* intellectual property rights of others.
//*--------------------------------------------------------------------------------------
//* File Name           : Audio.c
//* Object              : Audio function
//* Translator          :
//* 1.0 25/11/02   NL   : Creation
//* 1.1 27/Apr/05  JPP  : Add new feature
//*--------------------------------------------------------------------------------------

#include "board.h"
#include "Audio.h"
#include "lib_pwm.h"

#include <math.h>
#define _pi_       3.14159265358979323846
#define SINUS_NB_PT  128
#define OFSSET       0x80
unsigned char sinus[SINUS_NB_PT];
double   pt_sinus;
double   index_setp;
double   ramp;

char Constant = (SAMPLE_SIZE >> 1);

AT91S_PWMC_PDC OutputSample;

//*----------------------------------------------------------------------------
//* \fn    SinusInit
//* \brief Set sinus param
//*----------------------------------------------------------------------------
void SinusInit(void)
{
    unsigned int cmpt;
    double sin_val;
    for (cmpt = 0; cmpt < SINUS_NB_PT ;cmpt++)
    {
     //* compute sinus  by point
         sin_val = (sin((2*_pi_)*((double)cmpt/(double)SINUS_NB_PT))*127);
         sinus[cmpt]=  sin_val  ;
         // set offset
         sinus[cmpt] += OFSSET  ;
    }
}

//*----------------------------------------------------------------------------
//* \fn    SetConstant
//* \brief Set DC Value
//*----------------------------------------------------------------------------
void SetConstant(char val)
{
   Constant= val;
}
//*----------------------------------------------------------------------------
//* \fn    OutputConstant
//* \brief Write DC value
//*----------------------------------------------------------------------------
__ramfunc void OutputConstant(char *ptr_data,int size)
{
   unsigned int i;
   for (i=0 ; i<size; i++)
   {
	ptr_data[i]= Constant;
   }
}
//*----------------------------------------------------------------------------
//* \fn    SetSinus
//* \brief Set sinus param
//*----------------------------------------------------------------------------
void SetSinus(unsigned int freq )
{
   index_setp =  (double)SINUS_NB_PT / (double)( (double)SAMPLE_FREQUENCY / (double)freq );
   pt_sinus = 0;
}

//*----------------------------------------------------------------------------
//* \fn    OutputSinus
//* \brief Write sinus in output buffer
//*----------------------------------------------------------------------------
__ramfunc void OutputSinus(char *ptr_data,int size)
{
   unsigned int i;
	for (i=0 ; i<size; i++)
	{
		ptr_data[i]= sinus[ (int)(pt_sinus)& 0xff ];
		pt_sinus +=index_setp;
        if (pt_sinus > (SINUS_NB_PT - 1))  pt_sinus = 0;
    }
}
//*----------------------------------------------------------------------------
//* \fn    SetRamp
//* \brief Set ram param
//*----------------------------------------------------------------------------
void SetRamp(unsigned int freq )
{
   index_setp = (double)255 / (double)( (double)SAMPLE_FREQUENCY / (double)freq );
   ramp = 0;
}
//*----------------------------------------------------------------------------
//* \fn    OutputRamp
//* \brief Set ramp in output buffer
//*----------------------------------------------------------------------------
__ramfunc void OutputRamp(char *ptr_data,int nb_output)
{
    unsigned int i;
    for (i=0 ; i < nb_output; i++)
    {
         ptr_data[i]= (char) ramp  ;
         ramp += index_setp;
         if (ramp > (SAMPLE_SIZE-1))  ramp =0;
    }
}
//*----------------------------------------------------------------------------
//* \fn    AT91F_PWM_OutputSample_init
//* \brief Enable PWM channel 0 device
//*----------------------------------------------------------------------------
void OutputSample_init(AT91F_AUDIO_FUNCTION func)
{
    OutputSample.func = func;
    OutputSample.index = 0;
    OutputSample.state = INDEX_MAX;
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_PWM_OutputSample_init
//* \brief Enable PWM channel 0 device
//*----------------------------------------------------------------------------
void AT91F_PWM_OutputSample_init(void)
{
    OutputSample_init(OutputConstant);
}
